﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO.Ports;
using Modbus.Device;
using Modbus.IO;
using Modbus.Utility;
using System.Runtime.ConstrainedExecution;
using System.Threading;

namespace ZLAC8015_RS485_GUI
{
    class Modbus
    {
        SerialPort _port;
        ModbusSerialMaster _master;

        public readonly ushort Control_word_ADDR = 0x2031;
        public readonly ushort Operating_mode_ADDR = 0x2032;

        public enum OP_Mode
        {
            Position_ABS = 0x01,
            Position_REL,
            Velocity,
            Torque
        }
        public enum Control_word
        {
            ALM_CLR = 0x06,
            Servo_Off,
            Servo_On,
            Start = 0x10
        }

        public enum Read_Data
        {
            Motor_Temperature = 0x2026,
            Motor_Status,
            Hall_Status,
            Bus_Voltage,
            Position_High,
            Position_Low,
            Motor_Speed,
            Motor_Torque,
            Error_Code
        }
        public enum Read_Data_Mapping
        {
            Motor_Temperature,
            Motor_Status,
            Hall_Status,
            Bus_Voltage,
            Position_High,
            Position_Low,
            Motor_Speed,
            Motor_Torque,
            Error_Code
        }



        /// <summary>
        /// 포트 연결
        /// </summary>
        /// <returns> 연결 완료시 true</returns>
        public bool Connect(String Comport, int Baudrate)
        {
            string portName = Comport;
            _port = new SerialPort(portName, Baudrate);
            _port.ReadTimeout = 100;
            _port.WriteTimeout = 100;
            if (!_port.IsOpen)
            {
                _port.Open();
                // Initialize Modbus master
                _master = ModbusSerialMaster.CreateRtu(_port); // ModbusRtu Master 설정
                _master.Transport.Retries = 0;
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// 포트 연결 해제
        /// </summary>
        /// <returns>연결 해제 완료시 true 리턴</returns>
        public bool DisConnect()
        {
            if (_port.IsOpen)
            {
                _port.Close();
                return true;
            }
            else
                return false;
        }
        /// <summary>
        /// 모터 서보 On
        /// ADDR 0x2031 Write 0x08
        /// </summary>
        /// <param name="Driver_ID">드라이버 ID</param>
        public void Servo_On(Byte Driver_ID)
        {
            if (_port.IsOpen)
            {
                _master.WriteSingleRegister(Driver_ID,Control_word_ADDR, (ushort)Control_word.Servo_On);
            }
            else return;
        }

        /// <summary>
        /// 모터 서보 Off
        /// </summary>
        /// <param name="Driver_ID">드라이버 ID</param>
        public void Servo_Off(Byte Driver_ID)
        {
            if (_port.IsOpen)
            {
                _master.WriteSingleRegister(Driver_ID, Control_word_ADDR, (ushort)Control_word.Servo_Off);
            }
            else return;
        }
        /// <summary>
        /// 위치 모드 
        /// </summary>
        /// <param name="Driver_ID">드라이버 ID</param>
        /// <param name="Position">위치 값</param>
        /// <param name="Cmd">Command</param>
        /// <param name="ACC">가속도 값</param>
        /// <param name="DEC">감속도 값</param>
        /// <param name="Speed">속도 값</param>
        public void Position_Motion(Byte Driver_ID, Int32 Position, UInt16 Cmd, UInt16 ACC, UInt16 DEC, UInt16 Speed)
        {
            Byte[] value = BitConverter.GetBytes(Position);
            ushort[] Set_value = new ushort[7];

            Set_value[0] = Cmd;
            Set_value[1] = 0;
            Set_value[2] = BitConverter.ToUInt16(value, 2);
            Set_value[3] = BitConverter.ToUInt16(value, 0);
            Set_value[4] = Speed;
            Set_value[5] = ACC;
            Set_value[6] = DEC;

            if (_port.IsOpen)
            {
                _master.WriteSingleRegister(Driver_ID, Operating_mode_ADDR, Set_value[0]);
                _master.WriteMultipleRegisters(Driver_ID, Operating_mode_ADDR, Set_value);
                
                _master.WriteSingleRegister(Driver_ID, Control_word_ADDR, (ushort)Control_word.Start);
            }
        }
        /// <summary>
        /// 속도 모드
        /// </summary>
        /// <param name="Driver_ID">드라이버 ID</param>
        /// <param name="ACC">가속도 값</param>
        /// <param name="DEC">감속도 값</param>
        /// <param name="Speed">속도 값</param>
        public void Velocity_Motion(Byte Driver_ID, UInt16 ACC, UInt16 DEC, Int16 Speed)
        {
            Byte[] value = BitConverter.GetBytes(Speed);
            ushort[] Set_value = new ushort[9];

            Set_value[0] = (ushort)OP_Mode.Velocity;
            Set_value[1] = 0;
            Set_value[2] = 0;
            Set_value[3] = 0;
            Set_value[4] = 0;
            Set_value[5] = ACC;
            Set_value[6] = DEC;
            Set_value[7] = 0;
            Set_value[8] = BitConverter.ToUInt16(value, 0);

            if(_port.IsOpen)
            {
                _master.WriteSingleRegister(Driver_ID, Operating_mode_ADDR, Set_value[0]);
                _master.WriteMultipleRegisters(Driver_ID, Operating_mode_ADDR, Set_value);
                //Thread.Sleep(10);
                _master.WriteSingleRegister(Driver_ID, Control_word_ADDR, (ushort)Control_word.Start);
            }
        }

        /// <summary>
        /// 토크 모드
        /// </summary>
        /// <param name="Driver_ID">드라이버 ID</param>
        /// <param name="Slope">가감속</param>
        /// <param name="Torque">토크</param>
        public void Torque_Motion(Byte Driver_ID, UInt16 Slope, Int16 Torque)
        {
            Byte[] value = BitConverter.GetBytes(Torque);

            if(_port.IsOpen)
            {
                _master.WriteSingleRegister(Driver_ID, Operating_mode_ADDR, (ushort)OP_Mode.Torque);
                _master.WriteSingleRegister(Driver_ID, 0x203B, Slope);
                _master.WriteSingleRegister(Driver_ID, 0x2033, BitConverter.ToUInt16(value,0));
            }
        }

        /// <summary>
        /// 알람 클리어
        /// </summary>
        /// <param name="Driver_ID">드라이버 ID</param>
        public void ALM_Clear(byte Driver_ID)
        {
            if (_port.IsOpen)
            {
                _master.WriteSingleRegister(Driver_ID, Control_word_ADDR ,(ushort)Control_word.ALM_CLR);
            }
            else return;
        }

        /// <summary>
        /// I/O파라미터 읽기
        /// </summary>
        /// <param name="Driver_ID">드라이버 ID</param>
        /// <returns></returns>
        public ushort[] Read_IO_Parameter(Byte Driver_ID)
        {
            ushort[] Read_Value = new ushort[9];

            if (_port.IsOpen)
            {
                Read_Value = _master.ReadHoldingRegisters(Driver_ID, 0x2026, 9);
            }
            return Read_Value;
        }

        /// <summary>
        /// 위치값 초기화
        /// </summary>
        /// <param name="Driver_ID">드라이버 ID</param>
        public void Position_Reset (Byte Driver_ID)
        {
            if (_port.IsOpen)
            {
                _master.WriteSingleRegister(Driver_ID, 0x2005, 0x01);
                _master.WriteSingleRegister(Driver_ID, 0x2006, 0x01);
            }
            
        }
       
        /// <summary>
        /// 위치모드중 정지 -> Servo OFF
        /// </summary>
        /// <param name="Driver_ID"></param>
        public void Position_Stop(Byte Driver_ID)
        {
            if (_port.IsOpen)
            {
                _master.WriteSingleRegister(Driver_ID, Control_word_ADDR, (ushort)Control_word.Servo_Off);
            }
        }
        /// <summary>
        /// 속도모드중 정지
        /// </summary>
        /// <param name="Driver_ID">드라이버 ID</param>
        public void Velocity_Stop(Byte Driver_ID)
        {
            if (_port.IsOpen)
            {
                _master.WriteSingleRegister(Driver_ID,0x203A, 0x00);
            }
        }
        /// <summary>
        /// 토크모드중 정지
        /// </summary>
        /// <param name="Driver_ID">드라이버 ID</param>
        public void Torque_Stop(Byte Driver_ID)
        {
            if (_port.IsOpen)
            {
                _master.WriteSingleRegister(Driver_ID, 0x2033, 0x00);
            }
        }
        


    }
}